; ------------------------------------------------------------------
; --  _____       ______  _____                                    -
; -- |_   _|     |  ____|/ ____|                                   -
; --   | |  _ __ | |__  | (___    Institute of Embedded Systems    -
; --   | | | '_ \|  __|  \___ \   Zurich University of             -
; --  _| |_| | | | |____ ____) |  Applied Sciences                 -
; -- |_____|_| |_|______|_____/   8401 Winterthur, Switzerland     -
; ------------------------------------------------------------------
; --
; -- main.s
; --
; -- CT1 P08 "Strukturierte Codierung" mit Assembler
; --
; -- $Id: struct_code.s 3787 2016-11-17 09:41:48Z kesr $
; ------------------------------------------------------------------
;Directives
        PRESERVE8
        THUMB

; ------------------------------------------------------------------
; -- Address-Defines
; ------------------------------------------------------------------
; input
ADDR_DIP_SWITCH_7_0       EQU        0x60000200
ADDR_BUTTONS              EQU        0x60000210

; output
ADDR_LED_31_0             EQU        0x60000100
ADDR_7_SEG_BIN_DS3_0      EQU        0x60000114
ADDR_LCD_COLOUR           EQU        0x60000340
ADDR_LCD_ASCII            EQU        0x60000300
ADDR_LCD_ASCII_BIT_POS    EQU        0x60000302
ADDR_LCD_ASCII_2ND_LINE   EQU        0x60000314


; ------------------------------------------------------------------
; -- Program-Defines
; ------------------------------------------------------------------
; value for clearing lcd
ASCII_DIGIT_CLEAR        EQU         0x00000000
LCD_LAST_OFFSET          EQU         0x00000028

; offset for showing the digit in the lcd
ASCII_DIGIT_OFFSET        EQU        0x00000030

; lcd background colors to be written
DISPLAY_COLOUR_RED        EQU        0
DISPLAY_COLOUR_GREEN      EQU        2
DISPLAY_COLOUR_BLUE       EQU        4

; ------------------------------------------------------------------
; -- myConstants
; ------------------------------------------------------------------
        AREA myConstants, DATA, READONLY
; display defines for hex / dec
DISPLAY_BIT               DCB        "Bit "
DISPLAY_2_BIT             DCB        "2"
DISPLAY_4_BIT             DCB        "4"
DISPLAY_8_BIT             DCB        "8"
        ALIGN

; ------------------------------------------------------------------
; -- myCode
; ------------------------------------------------------------------
        AREA myCode, CODE, READONLY
        ENTRY

        ; imports for calls
        import adc_init
        import adc_get_value

main    PROC
        export main
        ; 8 bit resolution, cont. sampling
        BL         adc_init 
        BL         clear_lcd

main_loop
; STUDENTS: To be programmed
		
		; Read ADC Value to R0
		BL			adc_get_value
		
		; check T0
		LDR			R1, =ADDR_BUTTONS
		LDRB		R1, [R1]
		LDR			R2, =0x01
		TST     	R1, R2
		BNE			case_green					; If T0 is pressed: goto case_green, else: continue

		; Else (T0 is not pressed):
		
		; Read 8-bit value from DIP-Switch 7-0 to R1
		LDR			R1, =ADDR_DIP_SWITCH_7_0
		LDRB		R1, [R1]
		
		; Calc diff: DIP-Switch-7-0 - ADC-Value
		SUBS		R2, R1, R0
		BLT			case_red					; IF diff >= 0: continue case blue, else: goto case red
		

case_blue
		; Make LCD Display Blue
		LDR			R3, =ADDR_LCD_COLOUR
		LDR			R4, =DISPLAY_COLOUR_BLUE
		LDR			R5, =0xffff
		STRH		R5, [R3, R4]
		LDR			R4, =DISPLAY_COLOUR_RED
		LDR			R5, =0x0000
		STRH		R5, [R3, R4]
		LDR			R4, =DISPLAY_COLOUR_GREEN
		LDR			R5, =0x0000
		STRH		R5, [R3, R4]
		
		; Show Diff on 7-Segment
		LDR			R3, =ADDR_7_SEG_BIN_DS3_0
		STRB		R2, [R3]

		B			end_cases
case_red
		; Make LCD Display Red
		LDR			R3, =ADDR_LCD_COLOUR
		LDR			R4, =DISPLAY_COLOUR_RED
		LDR			R5, =0xffff
		STRH		R5, [R3, R4]
		LDR			R4, =DISPLAY_COLOUR_BLUE
		LDR			R5, =0x0000
		STRH		R5, [R3, R4]
		LDR			R4, =DISPLAY_COLOUR_GREEN
		LDR			R5, =0x0000
		STRH		R5, [R3, R4]
		
		; Show Diff on 7-Segment
		LDR			R3, =ADDR_7_SEG_BIN_DS3_0
		STRB		R2, [R3]

		B			end_cases
case_green
		; Make LCD Display Green
		LDR			R3, =ADDR_LCD_COLOUR
		LDR			R4, =DISPLAY_COLOUR_GREEN
		LDR			R5, =0xffff
		STRH		R5, [R3, R4]
		LDR			R4, =DISPLAY_COLOUR_RED
		LDR			R5, =0x0000
		STRH		R5, [R3, R4]
		LDR			R4, =DISPLAY_COLOUR_BLUE
		LDR			R5, =0x0000
		STRH		R5, [R3, R4]
		
		; Show ADC Value on 7-Semgment
		LDR			R3, =ADDR_7_SEG_BIN_DS3_0
		STRB		R0, [R3]

end_cases
; END: To be programmed
        B          main_loop
        
clear_lcd
        PUSH       {R0, R1, R2}
        LDR        R2, =0x0
clear_lcd_loop
        LDR        R0, =ADDR_LCD_ASCII
        ADDS       R0, R0, R2                       ; add index to lcd offset
        LDR        R1, =ASCII_DIGIT_CLEAR
        STR        R1, [R0]
        ADDS       R2, R2, #4                       ; increas index by 4 (word step)
        CMP        R2, #LCD_LAST_OFFSET             ; until index reached last lcd point
        BMI        clear_lcd_loop
        POP        {R0, R1, R2}
        BX         LR

write_bit_ascii
        PUSH       {R0, R1}
        LDR        R0, =ADDR_LCD_ASCII_BIT_POS 
        LDR        R1, =DISPLAY_BIT
        LDR        R1, [R1]
        STR        R1, [R0]
        POP        {R0, R1}
        BX         LR

        ENDP
        ALIGN


; ------------------------------------------------------------------
; End of code
; ------------------------------------------------------------------
        END
